﻿using Chenfeng.MES.Archiver.Core;
using Chenfeng.MES.Archiver.Data;
using CliFx;
using CliFx.Attributes;
using CliFx.Infrastructure;
using Dapper;
using MySqlConnector;
using System.Data;
using System.Diagnostics;
using System.Text;

namespace Chenfeng.MES.Archiver.Commands
{
    [Command("optimize")]
    public class OptimizeTableCommand : ICommand, IArchiveReader
    {
        [CommandParameter(0, Description = "数据库连接")]
        public string Connection { get; set; } = "";

        public MySqlConnection? Db { get; private set; }

        public Action<string> Print { get; set; } = (text) => { };

        [CommandOption("legacy", Description = "旧版MES")]
        public bool Legacy { get; set; } = false;

        [CommandOption("timeout", Description = "sql命令超时时间")]
        public int Timeout { get; set; } = 0;



        public ValueTask ExecuteAsync(IConsole console)
        {
            var startTime = DateTime.Now;
            Print = (text) => console.Output.WriteLine($"{DateTime.Now.ToLongTimeString()} {text}");

            var connBuilder = new MySqlConnectionStringBuilder(this.Connection);
            connBuilder.SslMode = MySqlSslMode.None;
            connBuilder.CharacterSet = "utf8";

            var starMonth = DateTime.Now.AddMonths(-12).ToString("yyyyMM");

            SqlMapper.Settings.CommandTimeout = 60 * Timeout;

            console.Output.WriteLine($"优化表数据");

            try
            {
                Db = new MySqlConnection(connBuilder.ConnectionString);

                Db.Open();
                if (Legacy)
                {
                    TableQueryHelper.MesLegacyTables(this, starMonth);
                }
                else
                {
                    TableQueryHelper.MesTables(this, starMonth);
                }


                Print("操作成功");

            }
            catch (Exception ex)
            {
                console.Output.WriteLine(ex.ToString());
            }
            finally
            {
                Db?.Close();
            }
            console.Output.WriteLine("执行耗时:" + (DateTime.Now - startTime).TotalMilliseconds + "ms");
            return default;
        }

        public void ExecuteData(string table, string where, object param)
        {
            var sql = $"alter table `{table}` engine = innodb ";
            Print("开始优化:" + table);
            Db.Execute(sql);
            Print("结束优化:" + table);
        }

        public void ExecuteChildData(string table, string parentTable, string condition)
        {
            ExecuteData(table, "", null);
        }


        public void Step(Action<string> action, params string[] tables)
        {
            foreach (var item in tables)
            {
                action(item);
            }
        }
    }
}
